#include <stdio.h>
#include <stdlib.h>
#include "my_queue.h"

void init(Queue *queue)
{
    queue->front = queue->rear = NULL;
    queue->length = 0;
}

bool is_full(const Queue *queue)
{
    return queue->length == MAXQUEUE;
}

bool is_empty(const Queue *queue)
{
    return queue->length == 0;
}

int queue_length(const Queue *queue)
{
    return queue->length;
}

bool enqueue(Item item, Queue *queue)
{
    if (is_full(queue))
        return false;

    Node *new_node = (Node *) malloc( sizeof(Node));
    if (!new_node)
    {
        fprintf(stderr,"Unable to allocate memory!\n");
        exit(1);
    }
    new_node->item = item;
    new_node->next = NULL;

    if (is_empty(queue))
        queue->front = new_node;
    else
        queue->rear->next = new_node;

    queue->rear = new_node; // remark the rear
    queue->length++;
    
    return true;
}

bool dequeue(Item *item_ptr, Queue *queue)
{
	if (is_empty(queue))
        return false;

    *item_ptr = queue->front->item;

    Node *temp = queue->front;
    queue->front = queue->front->next;
    free(temp);
    queue->length--;

    if (queue->length == 0)
        queue->rear = NULL;
    
    return true;
}

void traverse(const Queue *queue, void (*func_ptr)(Item item))
{
    Node *cursor = queue->front;
    while (cursor != NULL)
    {
        (*func_ptr)(cursor->item);
        cursor = cursor->next;
    }
}

void clear(Queue *queue)
{
    Item dummy;
    while (!is_empty(queue))
        dequeue(&dummy, queue);
}
